<?php
/**
 * Clean up wp_head output.
 *
 * @package Flat101\Security\Model\Block
 */

namespace Flat101\Security\Model\Block;

use Carbon_Fields\Field\Field;
use Flat101\Security\Model\Entity\Block;
use Flat101\Security\Model\Entity\Setting;

/**
 * CleanWpHeadBlock class.
 */
class CleanWpHeadBlock extends Block {

	/**
	 * Block constructor.
	 */
	public function __construct() {
		parent::__construct(
			esc_html__( 'Clean wp_head', 'flat101-security' ),
			'clean_wp_head',
			array(
				new Setting( 'remove_wp_version', false, esc_html__( 'Remove WP version', 'flat101-security' ) ),
				new Setting( 'remove_rsd_link', false, esc_html__( 'Remove Really Simple Discovery link', 'flat101-security' ) ),
				new Setting( 'remove_wlwmanifest_link', false, esc_html__( 'Remove Windows Live Writer manifest link', 'flat101-security' ) ),
				new Setting( 'remove_rest_api_link', false, esc_html__( 'Remove REST API link', 'flat101-security' ) ),
				new Setting( 'remove_oembed_links', false, esc_html__( 'Remove oEmbed discovery links', 'flat101-security' ) ),
				new Setting( 'remove_rel_links', false, esc_html__( 'Remove rel next/prev links', 'flat101-security' ) ),
				new Setting( 'remove_prefetch_url', false, esc_html__( 'Remove prefetch url', 'flat101-security' ) ),
				new Setting( 'remove_shortlink', false, esc_html__( 'Remove shortlink', 'flat101-security' ) ),
				new Setting( 'remove_all', false, esc_html__( 'Remove ALL', 'flat101-security' ) ),
			)
		);
	}


	// PHPDoc (inherited).
	// phpcs:ignore Squiz.Commenting.FunctionComment.Missing
	public function get_description() {
		return wp_kses_post(
			__(
				'This section allows you to clean up the wp_head output by removing unnecessary meta tags and links that
				are output by WordPress core and other plugins. Cleaning up the wp_head can help reduce the size of your page
				and make your HTML source code cleaner. This could potentially improve the loading time and the performance of
				your website<sup><a href="https://developer.wordpress.org/reference/functions/wp_head/" target="_blank">1</a></sup>.',
				'flat101-security'
			)
		);
	}


	// PHPDoc (inherited).
	// phpcs:ignore Squiz.Commenting.FunctionComment.Missing
	public function get_fields() {
		return array(
			Field::make( 'checkbox', $this->get_option_name( 'remove_all' ), $this->get_setting_label( 'remove_all' ) )
				->set_default_value( $this->get_setting_default_value( 'remove_all' ) ),

			Field::make( 'checkbox', $this->get_option_name( 'remove_wp_version' ), $this->get_setting_label( 'remove_wp_version' ) )
				->set_help_text( 'Removing the WP version meta tag from the header and the generator tag from feeds can help obscure the version of WordPress you are using from potential attackers. This can be a beneficial step in a broader security strategy, as it may deter automated attacks targeting specific known vulnerabilities in certain WordPress versions.' )
				->set_default_value( $this->get_setting_default_value( 'remove_wp_version' ) ),

			Field::make( 'checkbox', $this->get_option_name( 'remove_rsd_link' ), $this->get_setting_label( 'remove_rsd_link' ) )
				->set_help_text( 'The Really Simple Discovery (RSD) link is used by some blogging clients to discover the endpoint of blogging APIs. However, if you do not use such a client or API, this link is unnecessary and removing it can help clean up your site\'s header output, potentially improving the performance and loading time of your site.' )
				->set_default_value( $this->get_setting_default_value( 'remove_rsd_link' ) ),

			Field::make( 'checkbox', $this->get_option_name( 'remove_wlwmanifest_link' ), $this->get_setting_label( 'remove_wlwmanifest_link' ) )
				->set_help_text( 'The Windows Live Writer manifest link is used for a now-defunct blogging tool from Microsoft. There is virtually no use for this link in modern WordPress sites, and removing it can help clean up your site\'s header output, potentially improving the performance and loading time of your site.' )
				->set_default_value( $this->get_setting_default_value( 'remove_wlwmanifest_link' ) ),

			Field::make( 'checkbox', $this->get_option_name( 'remove_rest_api_link' ), $this->get_setting_label( 'remove_rest_api_link' ) )
				->set_help_text( 'The REST API link tag provides the endpoint for the WordPress REST API. If you are not using the REST API on your site, or if you prefer to keep the endpoint obscure, you can remove this link. This can also help clean up your site\'s header output, potentially improving the performance and loading time of your site.' )
				->set_default_value( $this->get_setting_default_value( 'remove_rest_api_link' ) ),

			Field::make( 'checkbox', $this->get_option_name( 'remove_oembed_links' ), $this->get_setting_label( 'remove_oembed_links' ) )
				->set_help_text( 'oEmbed discovery links allow other sites to easily embed content from your site. If you do not want to allow others to embed your content, or if you do not have content that can be embedded, you can remove these links to clean up your site\'s header output, potentially improving the performance and loading time of your site.' )
				->set_default_value( $this->get_setting_default_value( 'remove_oembed_links' ) ),

			Field::make( 'checkbox', $this->get_option_name( 'remove_rel_links' ), $this->get_setting_label( 'remove_rel_links' ) )
				->set_help_text( 'Rel next/prev links are used to indicate the relationship between paginated pages. If your site does not use pagination, or if you want to clean up your site\'s header output, you can remove these links. This can potentially improve the performance and loading time of your site.' )
				->set_default_value( $this->get_setting_default_value( 'remove_rel_links' ) ),

			Field::make( 'checkbox', $this->get_option_name( 'remove_prefetch_url' ), $this->get_setting_label( 'remove_prefetch_url' ) )
				->set_help_text( 'Prefetch URLs are used to indicate resources that should be prefetched by the browser for performance reasons. If you want to have more control over what is prefetched or if you want to clean up your site\'s header output, you can remove these links. This can potentially improve the performance and loading time of your site.' )
				->set_default_value( $this->get_setting_default_value( 'remove_prefetch_url' ) ),

			Field::make( 'checkbox', $this->get_option_name( 'remove_shortlink' ), $this->get_setting_label( 'remove_shortlink' ) )
				->set_help_text( 'Shortlinks are shorter URLs for your pages and posts. While they can be handy, they are not always necessary. If you do not use shortlinks, you can remove the shortlink tag from your site\'s header output to clean it up, which can potentially improve the performance and loading time of your site.' )
				->set_default_value( $this->get_setting_default_value( 'remove_shortlink' ) ),
		);
	}

	// PHPDoc (inherited).
	// phpcs:ignore Squiz.Commenting.FunctionComment.Missing
	public function run() {

		$remove_all = $this->get_setting_value( 'remove_all' );

		if ( $remove_all || $this->get_setting_value( 'remove_wp_version' ) ) {
			remove_action( 'wp_head', 'wp_generator' );
			add_filter( 'the_generator', '__return_empty_string' );
		}

		if ( $remove_all || $this->get_setting_value( 'remove_rsd_link' ) ) {
			remove_action( 'wp_head', 'rsd_link' );
		}

		if ( $remove_all || $this->get_setting_value( 'remove_wlwmanifest_link' ) ) {
			remove_action( 'wp_head', 'wlwmanifest_link' );
		}

		if ( $remove_all || $this->get_setting_value( 'remove_rest_api_link' ) ) {
			remove_action( 'wp_head', 'rest_output_link_wp_head', 10 );
		}

		if ( $remove_all || $this->get_setting_value( 'remove_oembed_links' ) ) {
			remove_action( 'wp_head', 'wp_oembed_add_discovery_links', 10 );
		}

		if ( $remove_all || $this->get_setting_value( 'remove_rel_links' ) ) {
			remove_action( 'wp_head', 'adjacent_posts_rel_link', 10, 0 );
		}

		if ( $remove_all || $this->get_setting_value( 'remove_prefetch_url' ) ) {
			remove_action( 'wp_head', 'wp_resource_hints', 2 );
		}

		if ( $remove_all || $this->get_setting_value( 'remove_shortlink' ) ) {
			add_action(
				'after_setup_theme',
				function () {
					remove_action( 'wp_head', 'wp_shortlink_wp_head', 10 );
				}
			);
		}
	}
}
