<?php
/**
 * Disable RSS feeds.
 *
 * @package Flat101\Security\Model\Block
 */

namespace Flat101\Security\Model\Block;

use Carbon_Fields\Field\Field;
use Flat101\Security\Model\Entity\Block;
use Flat101\Security\Model\Entity\Setting;

/**
 * RemoveFeedsBlock class.
 */
class RemoveFeedsBlock extends Block {

	/**
	 * Block constructor.
	 */
	public function __construct() {
		parent::__construct(
			esc_html__( 'Remove Feeds', 'flat101-security' ),
			'remove_feeds',
			array(
				new Setting( 'disable_all_feeds', false, esc_html__( 'Disable all feeds', 'flat101-security' ) ),
				new Setting( 'disable_rdf', false, esc_html__( 'Disable RDF feed', 'flat101-security' ) ),
				new Setting( 'disable_rss', false, esc_html__( 'Disable RSS feed', 'flat101-security' ) ),
				new Setting( 'disable_rss2', false, esc_html__( 'Disable RSS2 feed', 'flat101-security' ) ),
				new Setting( 'disable_atom', false, esc_html__( 'Disable Atom feed', 'flat101-security' ) ),
			)
		);
	}

	// PHPDoc (inherited).
	// phpcs:ignore Squiz.Commenting.FunctionComment.Missing
	public function get_description() {
		return wp_kses_post(
			__(
				'Enabling these features will disable the specified RSS feeds on your WordPress site. This can help prevent
				content scraping and unauthorized distribution of your site\'s content.',
				'flat101-security'
			)
		);
	}

	// PHPDoc (inherited).
	// phpcs:ignore Squiz.Commenting.FunctionComment.Missing
	public function get_fields() {
		return array(
			Field::make( 'checkbox', $this->get_option_name( 'disable_all_feeds' ), $this->get_setting_label( 'disable_all_feeds' ) )
				->set_default_value( $this->get_setting_default_value( 'disable_all_feeds' ) ),

			Field::make( 'checkbox', $this->get_option_name( 'disable_rdf' ), $this->get_setting_label( 'disable_rdf' ) )
				->set_default_value( $this->get_setting_default_value( 'disable_rdf' ) ),

			Field::make( 'checkbox', $this->get_option_name( 'disable_rss' ), $this->get_setting_label( 'disable_rss' ) )
				->set_default_value( $this->get_setting_default_value( 'disable_rss' ) ),

			Field::make( 'checkbox', $this->get_option_name( 'disable_rss2' ), $this->get_setting_label( 'disable_rss2' ) )
				->set_default_value( $this->get_setting_default_value( 'disable_rss2' ) ),

			Field::make( 'checkbox', $this->get_option_name( 'disable_atom' ), $this->get_setting_label( 'disable_atom' ) )
				->set_default_value( $this->get_setting_default_value( 'disable_atom' ) ),
		);
	}

	// PHPDoc (inherited).
	// phpcs:ignore Squiz.Commenting.FunctionComment.Missing
	public function run() {
		// If "disable all feeds" is selected, add actions for all feed types.
		if ( $this->get_setting_value( 'disable_all_feeds' ) ) {
			$this->disable_all_feeds();
		} else {
			// Otherwise, check each feed type individually.
			if ( $this->get_setting_value( 'disable_rdf' ) ) {
				add_action( 'do_feed_rdf', array( $this, 'disable_feed' ), 1 );
			}
			if ( $this->get_setting_value( 'disable_rss' ) ) {
				add_action( 'do_feed_rss', array( $this, 'disable_feed' ), 1 );
			}
			if ( $this->get_setting_value( 'disable_rss2' ) ) {
				add_action( 'do_feed_rss2', array( $this, 'disable_feed' ), 1 );
			}
			if ( $this->get_setting_value( 'disable_atom' ) ) {
				add_action( 'do_feed_atom', array( $this, 'disable_feed' ), 1 );
			}
		}
	}

	/**
	 * Disable all RSS feed functionalities.
	 */
	private function disable_all_feeds() {
		// Add the action for all feed types.
		add_action( 'do_feed', array( $this, 'disable_feed' ), 1 );
		add_action( 'do_feed_rdf', array( $this, 'disable_feed' ), 1 );
		add_action( 'do_feed_rss', array( $this, 'disable_feed' ), 1 );
		add_action( 'do_feed_rss2', array( $this, 'disable_feed' ), 1 );
		add_action( 'do_feed_atom', array( $this, 'disable_feed' ), 1 );
	}

	/**
	 * The method that's hooked into the feed actions.
	 */
	public function disable_feed() {
		wp_die(
			wp_kses_post(
				sprintf(
					/* translators: %s: is a HTML link */
					__( 'Sorry, our content is not available through RSS. Please visit our <a href="%s">website</a> to read it.', 'flat101-security' ),
					esc_url( home_url( '/' ) )
				)
			)
		);
	}
}
